@@ -1,20 +1,20 @@
-use serde::{Serialize, Deserialize};
+use serde::{Deserialize, Serialize};
 use net2::TcpStreamExt;
 use rmp_serde;
 use rand::{self, Rand};
 
-use std::net::{TcpListener, TcpStream, ToSocketAddrs, Shutdown, SocketAddr};
-use std::sync::{RwLock, Arc, Mutex};
+use std::net::{Shutdown, SocketAddr, TcpListener, TcpStream, ToSocketAddrs};
+use std::sync::{Arc, Mutex, RwLock};
 use std::collections::HashMap;
 use std::ops::Deref;
 use std::fmt::Debug;
 use std::hash::Hash;
-use std::{mem, fmt};
+use std::{fmt, mem};
 use std::time::Duration;
 use std::thread::{self, JoinHandle};
-use std::io::{self, BufWriter};
+use std::io::BufWriter;
 
-use super::stats::{Stats, StatReader, StatWriter};
+use super::stats::{StatReader, StatWriter, Stats};
 use super::queue::Queue;
 
 
@@ -44,16 +44,18 @@ impl<T> InitMessage for T where T: Serialize + Deserialize + Send + Sync + Clone
 
 
 /// The error type used througout the crate
-#[derive(Debug)]
-pub enum Error<N> where N: NodeId {
+#[derive(Debug, PartialEq, Hash, Ord, Eq, PartialOrd)]
+pub enum Error<N>
+    where N: NodeId
+{
     /// The node has already been closed
     AlreadyClosed,
 
     /// Failed to open a node socket
-    OpenError(io::Error),
+    OpenError,
 
     /// Failed to establish a connection
-    ConnectionError(io::Error),
+    ConnectionError,
 
     /// Failed to send a message
     SendError,
@@ -68,7 +70,7 @@ pub enum Error<N> where N: NodeId {
     ConnectionAborted,
 
     /// Failed to close a socket
-    CloseError(io::Error)
+    CloseError,
 }
 
 
@@ -119,7 +121,7 @@ pub enum Event<M: Message, N: NodeId, I: InitMessage> {
     ///
     /// This event is emitted whenever the node has shut down. At this time, all connections
     /// have been closed and no final messages can be sent.
-    Closed
+    Closed,
 }
 
 
@@ -148,7 +150,7 @@ impl<M: Message, N: NodeId, I: InitMessage> Deref for CloseGuard<M, N, I> {
 /// Node statistics
 pub struct NodeStats<N: NodeId> {
     /// Statistics for all connections
-    pub connections: HashMap<N, ConnectionStats>
+    pub connections: HashMap<N, ConnectionStats>,
 }
 
 
@@ -204,7 +206,7 @@ impl<M: Message, N: NodeId, I: InitMessage> Node<M, N, I> {
     /// the first message sent automatically to other nodes when connected.
     /// The result of this call is a guard that closes the node if dropped.
     pub fn new(node_id: N, init_message: I) -> CloseGuard<M, N, I> {
-        CloseGuard(Node(Arc::new(NodeInner{
+        CloseGuard(Node(Arc::new(NodeInner {
             node_id: node_id,
             events: Queue::new(),
             sockets: Mutex::new(Vec::new()),
@@ -273,7 +275,7 @@ impl<M: Message, N: NodeId, I: InitMessage> Node<M, N, I> {
             return Err(Error::AlreadyClosed);
         }
         let mut nodes = self.sockets.lock().expect("Lock poisoned");
-        let node: Arc<TcpListener> = Arc::new(try!(TcpListener::bind(addr).map_err(|err| Error::OpenError(err))));
+        let node: Arc<TcpListener> = Arc::new(try!(TcpListener::bind(addr).map_err(|_| Error::OpenError)));
         let cloned_self = self.clone();
         let cloned_node = node.clone();
         let used_addr = node.local_addr().expect("Failed to get local address");
@@ -298,7 +300,8 @@ impl<M: Message, N: NodeId, I: InitMessage> Node<M, N, I> {
     /// The addresses of this node
     pub fn addresses(&self) -> Vec<SocketAddr> {
         let mut addrs = Vec::new();
-        for &(ref sock, _) in &self.sockets.lock().expect("Lock poisoned") as &Vec<(Arc<TcpListener>, _)> {
+        for &(ref sock, _) in
+            &self.sockets.lock().expect("Lock poisoned") as &Vec<(Arc<TcpListener>, _)> {
             addrs.push(sock.local_addr().expect("Failed to obtain address"));
         }
         addrs
@@ -313,7 +316,7 @@ impl<M: Message, N: NodeId, I: InitMessage> Node<M, N, I> {
     pub fn receive(&self) -> Event<M, N, I> {
         match self.events.get() {
             Some(evt) => evt,
-            None => Event::Closed
+            None => Event::Closed,
         }
     }
 
@@ -329,7 +332,7 @@ impl<M: Message, N: NodeId, I: InitMessage> Node<M, N, I> {
         match self.events.get_timeout(timeout) {
             Some(Some(evt)) => Some(evt),
             Some(None) => Some(Event::Closed),
-            None => None
+            None => None,
         }
     }
 
@@ -362,13 +365,18 @@ impl<M: Message, N: NodeId, I: InitMessage> Node<M, N, I> {
         self.connections.read().expect("Lock poisoned").contains_key(id)
     }
 
+    /// The number of fully established connections
+    pub fn connection_count(&self) -> usize {
+        self.connections.read().expect("Lock poisoned").len()
+    }
+
     fn get_connections(&self) -> Vec<Connection<M, N, I>> {
         self.connections.read().expect("Lock poisoned").values().map(|c| c.clone()).collect()
     }
 
     /// The statistics of this node
     pub fn stats(&self) -> NodeStats<N> {
-        let mut stats = NodeStats{connections: HashMap::new()};
+        let mut stats = NodeStats { connections: HashMap::new() };
         for (id, con) in self.connections.read().expect("Lock poisoned").iter() {
             stats.connections.insert(id.clone(), con.stats());
         }
@@ -377,7 +385,7 @@ impl<M: Message, N: NodeId, I: InitMessage> Node<M, N, I> {
 
     fn run_node(&self, socket: Arc<TcpListener>) -> Result<(), Error<N>> {
         loop {
-            let (sock, _) = try!(socket.accept().map_err(|e| Error::ConnectionError(e)));
+            let (sock, _) = try!(socket.accept().map_err(|_| Error::ConnectionError));
             let req = try!(ConnectionRequest::new(self.clone(), sock));
             self.events.put(Event::ConnectionRequest(req));
         }
@@ -396,7 +404,7 @@ impl<M: Message, N: NodeId, I: InitMessage> Node<M, N, I> {
         }
         match self.get_connection(dst) {
             Some(con) => con.send(msg),
-            None => Err(Error::NotConnected(dst.clone()))
+            None => Err(Error::NotConnected(dst.clone())),
         }
     }
 
@@ -413,7 +421,7 @@ impl<M: Message, N: NodeId, I: InitMessage> Node<M, N, I> {
         if *self.closed.read().expect("Lock poisoned") {
             return Err(Error::AlreadyClosed);
         }
-        let sock = try!(TcpStream::connect(addr).map_err(|err| Error::ConnectionError(err)));
+        let sock = try!(TcpStream::connect(addr).map_err(|_| Error::ConnectionError));
         Ok(try!(ConnectionRequest::new(self.clone(), sock)))
     }
 
@@ -437,10 +445,26 @@ impl<M: Message, N: NodeId, I: InitMessage> Node<M, N, I> {
         Ok(id)
     }
 
+    /// Returns the node id for an address if connected
+    ///
+    /// This method retrns the node id for an address if a connection to this address exists.
+    pub fn lookup_connection<A: ToSocketAddrs>(&self, addr: A) -> Option<N> {
+        let lock = self.connections.read().expect("Lock poisoned");
+        for (id, con) in &lock as &HashMap<N, Connection<M, N, I>> {
+            let sock_addr = con.get_address();
+            for a in addr.to_socket_addrs().unwrap() {
+                if a == sock_addr {
+                    return Some(id.clone());
+                }
+            }
+        }
+        None
+    }
+
     fn shutdown_socket(&self, socket: &TcpListener) -> Result<(), Error<N>> {
-        //TODO: Remove this workaround once a proper API is available
+        // TODO: Remove this workaround once a proper API is available
         let socket = unsafe { mem::transmute::<&TcpListener, &TcpStream>(socket) };
-        socket.shutdown(Shutdown::Both).map_err(|e| Error::CloseError(e))
+        socket.shutdown(Shutdown::Both).map_err(|_| Error::CloseError)
     }
 
     fn accept_connection(&self, con: Connection<M, N, I>) {
@@ -448,7 +472,7 @@ impl<M: Message, N: NodeId, I: InitMessage> Node<M, N, I> {
         thread::spawn(move || con.run());
     }
 
-    fn close(&self) -> Result<(), Error<N>> {
+    pub fn close(&self) -> Result<(), Error<N>> {
         self.events.put(Event::Closing);
         *self.closed.write().expect("Lock poisoned") = true;
         let mut sockets = self.sockets.lock().expect("Lock poisoned");
@@ -505,7 +529,7 @@ pub struct ConnectionStats {
     pub read_rate: f64,
 
     /// The current idle time of reading side
-    pub read_idle: Duration
+    pub read_idle: Duration,
 }
 
 
@@ -528,31 +552,43 @@ pub struct ConnectionRequest<M: Message, N: NodeId, I: InitMessage> {
 
 impl<M: Message, N: NodeId, I: InitMessage> PartialEq for ConnectionRequest<M, N, I> {
     fn eq(&self, other: &Self) -> bool {
-        self.node_id == other.node_id
-        && self.socket.peer_addr().unwrap() == other.socket.peer_addr().unwrap()
-        && self.socket.local_addr().unwrap() == other.socket.local_addr().unwrap()
+        self.node_id == other.node_id &&
+        self.socket.peer_addr().unwrap() == other.socket.peer_addr().unwrap() &&
+        self.socket.local_addr().unwrap() == other.socket.local_addr().unwrap()
     }
 }
 
 impl<M: Message, N: NodeId, I: InitMessage> fmt::Debug for ConnectionRequest<M, N, I> {
     fn fmt(&self, fmt: &mut fmt::Formatter) -> Result<(), fmt::Error> {
-        write!(fmt, "ConnectionRequest(from: {:?}, node_id: {:?}, init: {:?})", self.socket.peer_addr().unwrap(), self.node_id, self.init)
+        write!(fmt,
+               "ConnectionRequest(from: {:?}, node_id: {:?}, init: {:?})",
+               self.socket.peer_addr().unwrap(),
+               self.node_id,
+               self.init)
     }
 }
 
 impl<M: Message, N: NodeId, I: InitMessage> ConnectionRequest<M, N, I> {
     fn new(node: Node<M, N, I>, mut socket: TcpStream) -> Result<Self, Error<N>> {
-        try!(socket.set_nodelay(true).map_err(|err| Error::ConnectionError(err)));
-        try!(socket.set_read_timeout(Some(node.connection_timeout())).map_err(|err| Error::ConnectionError(err)));
+        try!(socket.set_nodelay(true).map_err(|_| Error::ConnectionError));
+        try!(socket.set_read_timeout(Some(node.connection_timeout()))
+                   .map_err(|_| Error::ConnectionError));
         {
             let mut writer = rmp_serde::Serializer::new(&mut socket);
-            try!((node.init_message(), node.node_id()).serialize(&mut writer).map_err(|_| Error::SendError));
+            try!((node.init_message(), node.node_id())
+                     .serialize(&mut writer)
+                     .map_err(|_| Error::SendError));
         }
         let (init, node_id) = {
             let mut reader = rmp_serde::Deserializer::new(&socket);
             try!(Deserialize::deserialize(&mut reader).map_err(|_| Error::ReadError))
         };
-        Ok(ConnectionRequest{node: node, socket: socket, init: init, node_id: node_id})
+        Ok(ConnectionRequest {
+            node: node,
+            socket: socket,
+            init: init,
+            node_id: node_id,
+        })
     }
 
     /// The initialization message received from the remote side.
@@ -588,7 +624,7 @@ pub struct ConnectionInner<M: Message, N: NodeId, I: InitMessage> {
     writer_stats: Arc<RwLock<Stats>>,
     reader: Mutex<rmp_serde::Deserializer<StatReader<TcpStream>>>,
     reader_stats: Arc<RwLock<Stats>>,
-    node_id: N
+    node_id: N,
 }
 
 
@@ -605,19 +641,21 @@ impl<M: Message, N: NodeId, I: InitMessage> Deref for Connection<M, N, I> {
 
 impl<M: Message, N: NodeId, I: InitMessage> Connection<M, N, I> {
     fn new(node: Node<M, N, I>, socket: TcpStream, node_id: N) -> Self {
-        let writer = StatWriter::new(socket.try_clone().expect("Failed to clone socket"), node.stats_halflife_time());
+        let writer = StatWriter::new(socket.try_clone().expect("Failed to clone socket"),
+                                     node.stats_halflife_time());
         let writer_stats = writer.stats();
-        let input = StatReader::new(socket.try_clone().expect("Failed to clone socket"), node.stats_halflife_time());
+        let input = StatReader::new(socket.try_clone().expect("Failed to clone socket"),
+                                    node.stats_halflife_time());
         let reader_stats = input.stats();
         let reader = rmp_serde::Deserializer::new(input);
-        Connection(Arc::new(ConnectionInner{
+        Connection(Arc::new(ConnectionInner {
             node: node,
             writer: Mutex::new(writer),
             writer_stats: writer_stats,
             reader: Mutex::new(reader),
             reader_stats: reader_stats,
             socket: Mutex::new(socket),
-            node_id: node_id
+            node_id: node_id,
         }))
     }
 
@@ -625,16 +663,20 @@ impl<M: Message, N: NodeId, I: InitMessage> Connection<M, N, I> {
         &self.node_id
     }
 
+    fn get_address(&self) -> SocketAddr {
+        self.socket.lock().expect("Lock poisoned").peer_addr().unwrap()
+    }
+
     fn stats(&self) -> ConnectionStats {
         let reader_stats = self.reader_stats.read().expect("Lock poisoned");
         let writer_stats = self.writer_stats.read().expect("Lock poisoned");
-        ConnectionStats{
+        ConnectionStats {
             write_total: writer_stats.total(),
             write_rate: writer_stats.rate(),
             write_idle: writer_stats.idle_time(),
             read_total: reader_stats.total(),
             read_rate: reader_stats.rate(),
-            read_idle: reader_stats.idle_time()
+            read_idle: reader_stats.idle_time(),
         }
     }
 
@@ -660,6 +702,6 @@ impl<M: Message, N: NodeId, I: InitMessage> Connection<M, N, I> {
     }
 
     fn close(&self) -> Result<(), Error<N>> {
-        Ok(try!(self.socket.lock().expect("Lock poisoned").shutdown(Shutdown::Both).map_err(|err| Error::CloseError(err))))
+        Ok(try!(self.socket.lock().expect("Lock poisoned").shutdown(Shutdown::Both).map_err(|_| Error::CloseError)))
     }
 }
